<?php

namespace Pinpoint\Feefo\Cron\Feed\Sales;

/**
 * Class Feed
 * @package Pinpoint\Feefo\Model\Feed\Sales
 */
class Feed extends \Magento\Framework\Model\AbstractModel
{

    /**
     * @var string
     */
    const ENDPOINT_URL = "https://api.feefo.com/api/entersaleremotely";

    /**
     * @var string
     */
    const MERCHANT_IDENTIFIER_CONFIG_PATH = 'pinpoint_feefo/account_details/merchant_identifier';

    /**
     * @var string
     */
    const ORDER_STATUS_CONFIG_PATH = 'pinpoint_feefo/feefo_import_settings/order_status';

    /**
     * @var string
     */
    const PRODUCT_ATTRIBUTES_CONFIG_PATH = 'pinpoint_feefo/feefo_import_settings/product_attributes';

    /**
     * @var string
     */
    const EMAIL_DELAY_CONFIG_PATH = 'pinpoint_feefo/feefo_import_settings/feedback_delay';

    /**
     * @var string
     */
    const ALT_GTIN_CONFIG_PATH = 'pinpoint_feefo/feefo_import_settings/use_alternative_gtin';

    /**
     * @var string
     */
    const ALT_MPN_CONFIG_PATH = 'pinpoint_feefo/feefo_import_settings/use_alternative_mpn';

    /**
     * @var int
     */
    const SKIP_PRODUCT = 0;

    /**
     * @var \Pinpoint\Feefo\Helper\Data
     */
    protected $feefoHelper;

    /**
     * @var \Magento\Sales\Model\ResourceModel\Order\CollectionFactory
     */
    protected $orderCollection;

    /**
     * @var \Magento\Store\Model\StoreManagerInterface
     */
    protected $storeManager;

    /**
     * @var \Magento\Framework\Filesystem\DirectoryList
     */
    protected $directoryList;

    /**
     * @var \Magento\Framework\Filesystem\Io\File
     */
    protected $ioFile;

    /**
     * Feed constructor.
     * @param \Magento\Sales\Model\ResourceModel\Order\CollectionFactory $orderCollectionFactory
     * @param \Pinpoint\Feefo\Helper\Data $feefoHelper
     * @param \Magento\Store\Model\StoreManagerInterface $storeManager
     * @param \Magento\Framework\App\State $appState
     * @param \Magento\Framework\Filesystem\DirectoryList $directoryList
     * @param \Magento\Framework\Filesystem\Io\File $ioFile
     * @param \Magento\Framework\Model\Context $context
     * @param \Magento\Framework\Registry $registry
     * @param \Magento\Framework\Model\ResourceModel\AbstractResource|null $resource
     * @param \Magento\Framework\Data\Collection\AbstractDb|null $resourceCollection
     * @param array $data
     */
    public function __construct(
        \Magento\Sales\Model\ResourceModel\Order\CollectionFactory $orderCollectionFactory,
        \Pinpoint\Feefo\Helper\Data $feefoHelper,
        \Magento\Store\Model\StoreManagerInterface $storeManager,
        \Magento\Framework\App\State $appState,
        \Magento\Framework\Filesystem\DirectoryList $directoryList,
        \Magento\Framework\Filesystem\Io\File $ioFile,
        \Magento\Framework\Model\Context $context,
        \Magento\Framework\Registry $registry,
        \Magento\Framework\Model\ResourceModel\AbstractResource $resource = null,
        \Magento\Framework\Data\Collection\AbstractDb $resourceCollection = null,
        array $data = []
    ) {
        $this->feefoHelper = $feefoHelper;
        $this->orderCollection = $orderCollectionFactory;
        $this->storeManager = $storeManager;
        $this->directoryList = $directoryList;
        $this->ioFile = $ioFile;

        try {
            $appState->setAreaCode(\Magento\Framework\App\Area::AREA_ADMINHTML);
        } catch(\Magento\Framework\Exception\LocalizedException $ex) {}

        parent::__construct($context, $registry, $resource, $resourceCollection, $data);
    }


    /**
     * generates sales feed for Feefo
     */
    public function generateFeed()
    {
        //get all orders from the past day that haven't been flagged to not be sent
        $orders = $this->orderCollection->create()
            ->addAttributeToSelect('*')
            ->addFieldToFilter(
                'status',
                ['in' => $this->feefoHelper->getConfig(self::ORDER_STATUS_CONFIG_PATH)]
            )
            ->addFieldToFilter('created_at', ['from' => date('Y-m-d 00:00:01', time()), 'to' => date('Y-m-d H:i:s', time())])
            ->addFieldToFilter('send_to_feefo', '1')
        ;

        //are we using a custom attribute?
        $gtin_attribute_code = $this->feefoHelper->getConfig(self::ALT_GTIN_CONFIG_PATH);
        if (!$gtin_attribute_code) {
            $gtin_attribute_code = 'pinpoint_gtin';
        }

        $mpn_attribute_code = $this->feefoHelper->getConfig(self::ALT_MPN_CONFIG_PATH);
        if (!$mpn_attribute_code) {
            $mpn_attribute_code = 'pinpoint_mpn';
        }

        //if we have orders to process
        if ($orders->count()) {
            $feed = ['items' => []];

            /** @var \Magento\Sales\Model\Order $order */
            foreach ($orders as $order) {
                $orderItems = $order->getAllVisibleItems();

                /** @var \Magento\Sales\Model\Order\Item $orderItem */
                foreach ($orderItems as $orderItem) {
                    //check to see if we should skip this order
                    if ($this->getCustomAttribute($orderItem, 'pinpoint_exclude_feefo') === self::SKIP_PRODUCT) {
                        continue;
                    }

                    //build order item category list
                    $categories = '';
                    foreach ($orderItem->getProduct()->getCategoryCollection() as $category) {
                        if ($category->getName()) {
                            $categories .= $category->getName() . ", ";
                        }
                    }

                    $orderTags = [
                        'store' => explode("\n", $order->getStoreName())[0],
                        'department' => $categories,
                        'saleschannel' => $order->getData('pinpoint_sales_channel'),
                        'salesperson' => $order->getData('pinpoint_sales_person')
                    ];

                    $tags = '';
                    foreach ($orderTags as $key => $value) {
                        if ($value == '') continue;

                        $tags .= sprintf("%s=%s,", $key, $value);
                    }

                    $parentSku = null;
                    if ($orderItem->getProductType() == \Magento\Catalog\Model\Product\Type::TYPE_SIMPLE) {
                        if ($orderItem->getParentItemId()) {
                            $parentSku = $orderItem->getParentItem()->getSku();
                        }
                    }

                    //build item xml structure to add to the xml feed
                    $feed['items'][] = [
                        'merchant_identifier' => $this->feefoHelper->getConfig(self::MERCHANT_IDENTIFIER_CONFIG_PATH),
                        'name' => $this->getNameFromOrder($order),
                        'email' => $order->getCustomerEmail(),
                        'date' => $order->getCreatedAt(),
                        'mobile' => $this->convertToUkNumber($order->getBillingAddress()->getTelephone()),
                        'description' => $this->getProductDescription($orderItem),
                        'order_ref' => $order->getIncrementId(),
                        'product_search_code' => $orderItem->getSku(),
                        'gtin' => $this->getCustomAttribute($orderItem, $gtin_attribute_code),
                        'mpn' => $this->getCustomAttribute($orderItem, $mpn_attribute_code),
                        'parent_product_ref' => $parentSku,
                        'feedback_date' => date('d/m/Y', strtotime(sprintf(
                            "%s +%s days",
                            $order->getUpdatedAt(),
                            $this->feefoHelper->getConfig(self::EMAIL_DELAY_CONFIG_PATH)
                        ))),
                        'customer_ref' => $order->getCustomerId(),
                        'amount' => $orderItem->getRowTotal(),
                        'currency' => $order->getOrderCurrencyCode(),
                        'product_link' => $orderItem->getProduct()->getProductUrl(),
                        'tags' => $tags,
                        'product_attributes' => $this->feefoHelper->getConfig(self::PRODUCT_ATTRIBUTES_CONFIG_PATH)
                    ];

                }
            }

            $this->buildAndSaveXml($feed['items']); //add order items to xml file
        }
    }

    /**
     * @param array $data
     * @return bool
     */
    protected function buildAndSaveXml(array $data)
    {
        $xml = new \SimpleXMLElement('<?xml version="1.0"?><items></items>');
        $this->arrayToXml($data, $xml);

        $export_path = $this->directoryList->getPath('var') . DIRECTORY_SEPARATOR . 'export' . DIRECTORY_SEPARATOR . 'feefo' . DIRECTORY_SEPARATOR;

        try {
            if (!\is_dir($export_path)) {
                $this->ioFile->mkdir($export_path);
            }

            $xml->asXML($export_path . 'feefo_orders_feed.xml');

            return true;
        } catch (\Exception $ex) {}

        return false;
    }

    /**
     * @param $data
     * @param $xml_data
     */
    protected function arrayToXml($data, &$xml_data) {
        foreach ($data as $key => $value) {
            if (is_numeric($key)) {
                $key = 'item';
            }
            if ($value && !empty($value)) {
                if (is_array($value)) {
                    $subnode = $xml_data->addChild($key);
                    $this->arrayToXml($value, $subnode);
                } else {
                    $xml_data->addChild("$key", htmlspecialchars(trim(preg_replace('"/\r|\n/"', ' ', "$value"))));
                }
            }
        }
    }

    /**
     * @param $product
     * @param $key
     * @return null|string
     */
    protected function getCustomAttribute($product, $key)
    {
        if ($attr = $product->getProduct()->getCustomAttribute($key)) {
            return $attr->getValue();
        }

        return null;
    }

    /**
     * @param $telephone
     * @return string
     */
    private function convertToUkNumber($telephone)
    {
        if (strpos($telephone, '0') === 0 && strlen($telephone) === 11) {
            $telephone = '+44' . substr($telephone, 1);
        }

        return $telephone;
    }

    /**
     * @param $order
     * @return string
     */
    private function getNameFromOrder($order)
    {
        if ($order->getCustomerFirstname() && $order->getCustomerLastname()) {
            return sprintf("%s %s", $order->getCustomerFirstname(), $order->getCustomerLastname());
        } else if ($order->getBillingAddress() instanceof \Magento\Sales\Api\Data\OrderAddressInterface) {
            $billingAddress = $order->getBillingAddress();
            return sprintf("%s %s", $billingAddress->getFirstname(), $billingAddress->getLastname());
        }

        return '';
    }

    /**
     * @param $orderItem
     * @return string
     */
    private function getProductDescription($orderItem)
    {
        $description = $orderItem->getProduct()->getDescription()?:$orderItem->getProduct()->getShortDescription();
        return strip_tags($description);
    }

}
